@@ -1,8 +1,9 @@
 package WWW::Pusher::Client;
 # ABSTRACT: Laughably incomplete Perl client for Pusher WS API
-$WWW::Pusher::Client::VERSION = '0.02';
+$WWW::Pusher::Client::VERSION = '0.03';
 use strict;
 use warnings;
+use 5.010_001;
 use Moo;
 use JSON;
 use AnyEvent::WebSocket::Client;
@@ -10,12 +11,12 @@ use Digest::SHA qw(hmac_sha256_hex);
 
 
 has 'auth_key' => (
-    is => 'rw' ,
+    is => 'ro' ,
     required => 1
 );
 
 has 'secret' => (
-    is => 'rw',
+    is => 'ro',
     required => 1
 );
 
@@ -24,20 +25,10 @@ has 'channel' => (
     predicate => 'has_channel'
 );
 
-has 'scheme' => (
-    is => 'rw',
-    default => 'ws'
-);
-
-has 'port' => (
-    is => 'rw',
-    default => 80
-);
-
 has 'client' => (
-    is => 'rw',
+    is => 'ro',
     lazy => 1,
-    default => sub { shift->{client} // AnyEvent::WebSocket::Client->new }
+    default => sub { AnyEvent::WebSocket::Client->new }
 );
 
 has 'ws_url' => (
@@ -46,45 +37,58 @@ has 'ws_url' => (
     builder => sub {
         my $self = shift;
 
-        return $self->{scheme} . $self->{_pusher_base} . $self->{port}
-        . "/app/" . $self->{auth_key}
-        . "?protocol=" . $self->{_protocol}
-        . "&client=" . $self->{_client_name}
-        . "&version=" . $self->{_version}
+        return $self->_scheme . $self->_pusher_base . $self->_port
+          . "/app/" . $self->auth_key
+          . "?protocol=" . $self->_protocol
+          . "&client=" . $self->_client_name
+          . "&version=" . $self->_version
     }
 );
 
 has 'ws_conn' => (
-    is => 'rw',
+    is => 'ro',
     lazy => 1,
     builder => sub {
         my $self = shift;
-        return AnyEvent::WebSocket::Client->new->connect($self->ws_url)->recv;
+
+        # open the connection and immediately ->recv the condvar to
+        # return control back to us
+        return $self->client->connect($self->ws_url)->recv;
     }
 );
 
+has '_scheme' => (
+    is => 'ro',
+    default => sub { 'ws' }
+);
+
+has '_port' => (
+    is => 'ro',
+    default => sub { 80 }
+);
+
 has '_pusher_base' => (
     is => 'ro',
-    default => '://ws.pusherapp.com:'
+    default => sub { '://ws.pusherapp.com:' }
 );
 
 has '_protocol' => (
     is => 'ro',
-    default => 7
+    default => sub { 7 }
 );
 
 has '_client_name' => (
     is => 'ro',
-    default => 'perl-pusher-client'
+    default => sub { 'perl-pusher-client' }
 );
 
 has '_version' => (
     is => 'ro',
-    default => '0.001'
+    default => sub { '0.001' }
 );
 
 has '_socket_id' => (
-    is => 'rw',
+    is => 'rw'
 );
 
 
@@ -109,24 +113,33 @@ sub BUILD {
 
 
 sub subscribe {
-    my $self = shift;
-    my $data = {
-        channel => $self->channel
-    };
+    my ($self, $channel) = @_;
 
-    # Private channels need a key:signature in the auth key for
-    # acceptance.
-    if ($self->channel =~ /^private\-/) {
-        my $signature = $self->_socket_auth($self->channel);
-        $data->{auth} = $self->auth_key . ':' . $signature;
-    }
+    my $data = $self->_construct_private_auth_data($channel);
 
-    $self->ws_conn->send(to_json({
+    return $self->ws_conn->send(to_json({
         event => 'pusher:subscribe',
         data => $data
     }));
 }
 
+sub _construct_private_auth_data {
+    my ($self, $channel) = @_;
+    $channel //= $self->channel;
+
+    # Public channels only need the channel name in the payload
+    my $data = { channel => $channel };
+
+    # Private channels need a key:signature in the auth key for
+    # authorization
+    if ($channel =~ /^private-/) {
+        my $signature = $self->_socket_auth($channel);
+        $data->{auth} = $self->auth_key . ':' . $signature;
+    }
+
+    return $data;
+}
+
 sub _socket_auth {
     my ($self, $channel) = @_;
     die 'Missing socket_id, sorry...' unless $self->_socket_id;
@@ -163,7 +176,7 @@ WWW::Pusher::Client - Laughably incomplete Perl client for Pusher WS API
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -188,7 +201,10 @@ only the most recent channel as defaults.
 The main difference between this module and L<WWW::Pusher> is that
 this module enables you to subscribe to channels like the WebSocket
 Pusher clients allow you to do. L<WWW::Pusher> interacts with Pusher
-via its HTTP API, which doesn't allow for subscriptions.
+via its HTTP API, which doesn't allow for subscriptions. On the other
+hand, this module uses L</AnyEvent::WebSocket::Client> to join
+channels via websockets and receive real time messages in addition to
+triggering them.
 
 =head1 METHODS
 
@@ -223,12 +239,45 @@ Trigger an event & message on the currently subscribed channel.
 
 =head1 SEE ALSO
 
-WWW::Pusher
+Please see those modules/websites for more information related to this module.
+
+=over 4
+
+=item *
+
+L<WWW::Pusher|WWW::Pusher>
+
+=back
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+https://github.com/honeydew-sc/WWW-Pusher-Client/issues
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
 
 =head1 AUTHOR
 
 Daniel Gempesaw <gempesaw@gmail.com>
 
+=head1 CONTRIBUTORS
+
+=for stopwords Syohei YOSHIDA gempesaw
+
+=over 4
+
+=item *
+
+Syohei YOSHIDA <syohex@gmail.com>
+
+=item *
+
+gempesaw <gempesaw@gmail.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2015 by Daniel Gempesaw.
